#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
Plug-in to open GIMP's vectors in 'Inkscape'. For Linux and Windows.

History:
 Version 0.4.0 (2016/03/17) at 'http://gimpchat.com/viewtopic.php?f=9&p=188222#p188222'
    Tested with GIMP-2.8.16 and Python-2.7.11 on Linux.
    Tested with GIMP-2.8.16 and Python-2.7.10 on Windows 7.
    With translation possibility

 Version 0.4.2 (2016/06/16)
    -Offer to save the arguments of the plug-in if a change in the folders args (SVG & Inkscape).
        By making a folder in 'gimp.directory' called 'py_configs' & a list file.
    -One running instance only.
    -Produce a 'bail_out(msg)' function to streamline the code.
    -Better management of the progress bar.
    -Use the name given at the beginning to the SVG file for the render layer.
    -Protect the plug-in exit and advise the user if no SVG file found after Inkscape,
        or avoid rendering on a group layer (a crash).

License:
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    The GNU Public License is available at
    http://www.gnu.org/copyleft/gpl.html
'''

import sys, os
import gtk, gettext
import subprocess as sub


### global #####################################################################

### change it to your liking
GTI_menu_path = "<Vectors>/"

### plug-in filename (thanks 'Ofnuts')
whoiam = os.path.abspath(sys.argv[0])
whoiamName = whoiam.split(os.sep)[-1][:-3]

# if not launch within GIMP
def mssgBox(mess):
    flag = gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT
    msgBox = gtk.MessageDialog(None, flag, gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, mess)
    msgBox.run()
    msgBox.destroy()
try:
    from gimpfu import *
    from gimpshelf import shelf
except ImportError:
    print("Note: GIMP is needed, '%s' is a plug-in for it.\n"%whoiam)
    # launch from GUI? use mssgBox(text)
    mssgBox("Note: GIMP registering is needed, '%s' is a plug-in for it.\n"%whoiam)
    sys.exit(1)

# for locale characters in windows file system
def sys_file(f_name):
    sys_encoded = sys.getfilesystemencoding()
    if os.name == 'nt': coded = f_name.encode(sys_encoded)
    else: coded = f_name
    return(coded)

# initialize internationalization in a user-locale
locale_user_dir = os.path.join(os.path.dirname(whoiam), 'locale')
gettext.install( whoiamName, locale_user_dir, unicode=True )

# folder for among other things an arg. values file in user-GIMP
GTI_folder = sys_file(gimp.directory + os.sep + 'py_configs')
# if folder not there, create it
if not os.path.exists(GTI_folder): os.mkdir(GTI_folder)
# avoid multiple running with dict. shelf (problem here)

# Linux and Windows, management of args
GTI_file_shelf = sys_file(GTI_folder+os.sep+whoiamName+'.txt')
GTI_args = ["", "", 0, 3, True]
if not os.path.isfile(GTI_file_shelf):
    if os.name == 'nt':
        GTI_args[0] = "C:\Program Files\Inkscape\inkscape.exe"
        #sys.stderr = open("C:/temp/%s-debug.txt"%whoiamName,'a')
    else:
        GTI_args[0] = "/usr/bin/inkscape"
        #sys.stderr = open("/home/robert/%s-debug.txt"%whoiamName,'a')
    GTI_args[1] = os.path.expanduser("~/Desktop")
else:   # read file for args
    data = open(GTI_file_shelf, 'r')
    GTI_args = eval(data.read())
    data.close()

#sys.stdout=sys.stderr # So that they both go to the same file


### main function ##############################################################

def open_path_in_inkscape(image, layer, ink, outputFolder, sel, schel, rem):
    global GTI_file_shelf, GTI_args, whoiamName

    # avoid multiple running with dict. shelf
    if shelf.has_key(whoiamName) and shelf[whoiamName]:
        gimp.message(_("WARNING: a %s is already running! Aborting new running.")%whoiamName)
        gtk.main_quit()
        return

    ### utility functions
    def bail_out(msg):
        """
        Display a message in the 'Error console' and bail out
        """
        gimp.message(msg)
        image.undo_group_end()
        gimp.progress_init(_("Plug-in had input error!"))
        pdb.gimp_progress_update(0.2)
        shelf[whoiamName] = False

    def render_svg_layer(img, svg_f, sep):
        """
        Rendering a 'Scalable Vector Graphics' file on the image top layer
        """
        resol_x = img.resolution[0]
        # image = pdb.file_svg_load(filename, raw_filename, resolution, width, height, paths)
        fileImage = pdb.file_svg_load(svg_f, svg_f, resol_x, 0, 0, 2)
        pdb.gimp_edit_copy(fileImage.layers[0])
        pdb.gimp_image_delete(fileImage)
        # pasted probl (RuntimeError) with group layer like in ArrowsCreator
        if hasattr(img.layers[0],"layers"): # Type = Group layer
            layer = img.new_layer('GTI', img.width, img.height, 2*img.base_type+1, 100, 0)
            pdb.gimp_layer_set_offsets(layer, 0, 0)
            pasted = pdb.gimp_edit_paste(layer, True)
            group_bool = True
        else:
            pasted = pdb.gimp_edit_paste(img.layers[0], True)
            group_bool = False
        pdb.gimp_floating_sel_to_layer(pasted)
        # set name of this layer base on filename
        newname = _('Render of ') + svg_f.split(sep)[-1]
        img.layers[0].name = newname
        if group_bool: pdb.gimp_image_remove_layer(img, layer)
        return True

    ### start main function,
    image.undo_group_start()
    shelf[whoiamName] = True
    # in GIMP - armouring (if no vectors 'layer' it won't start)
    if os.access(sys_file(outputFolder), os.W_OK):
        # construct the filename of vector exchange file in 'utf-8'
        xchange_utf = outputFolder + os.sep + layer.name + ".svg"
        gimp.progress_init(_("Saving to '") + xchange_utf)
    else:
        bail_out(_("ERROR: folder '%s' isn't writable, plug-in has terminated.")\
            %outputFolder)
        return
    if os.name == 'nt': tail = "inkscape.exe"
    else: tail = "inkscape"
    if not (ink.endswith(tail) and os.access(ink, os.X_OK)) :
        bail_out(_("ERROR: executable '%s' pose problem, plug-in has terminated.")\
            %ink)
        return

    if [GTI_args[0], GTI_args[1]] == [ink, outputFolder]: changed = False
    else: changed = True
    GTI_args[0] = ink
    GTI_args[1] = outputFolder

    # export which path;
    if sel == 0:
        pdb.gimp_vectors_export_to_file(image, xchange_utf, None)
    elif sel == 1:
        pdb.gimp_vectors_export_to_file(image, xchange_utf, image.active_vectors)
    GTI_args[2] = sel

    # plus inkscape with filename, 'ink' should be ASCII. Same file after inkscape?
    gimp.progress_init(_("Connecting to inkscape..."))
    pdb.gimp_progress_update(0.5)
    xchange_sys = sys_file(xchange_utf) # for Python, Win file system not 'utf-8'
    file_befor = os.stat(xchange_sys)
    p = sub.Popen([ink, xchange_sys], stdin=sub.PIPE, stdout=sub.PIPE, stderr=sub.STDOUT)
    child_output, child_error = p.communicate()
    # In inkscape saving a XCF file is also possible?

    # after inkscape, GIMP import vectors and/or render.
    if os.access(xchange_sys, os.F_OK):
        file_after = os.stat(xchange_sys)
        if schel == 1 and file_befor != file_after:     # vectors
            gimp.vectors_import_from_file(image, xchange_utf, TRUE, TRUE)
        elif schel == 2 :   # render
            render_svg_layer(image, xchange_utf, os.sep)
        elif schel == 3 :   # both
            if file_befor != file_after:
                gimp.vectors_import_from_file(image, xchange_utf, TRUE, TRUE)
            render_svg_layer(image, xchange_utf, os.sep)

        # cleanup vector file?
        if rem :
            os.remove(xchange_sys)
    else: gimp.message(_("WARNING: no SVG file found! So nothing after Inkscape."))
    GTI_args[3] = schel
    GTI_args[4] = rem

    gimp.displays_flush()
    shelf[whoiamName] = False
    image.undo_group_end()
    gimp.progress_init(_("Plug-in end normally."))
    pdb.gimp_progress_update(1.0)

    ## manage the arguments file for changed folders
    if changed:
        # message for saving the args?
        if (not os.path.isfile(GTI_file_shelf)): args_mess = _("The argument file is missing.")
        else: args_mess = _("The folder arguments seem changed.")
        mess1 = args_mess + _(" If you want to register these\n")\
            +_("args left-click on YES. The file will be '%s'.")%GTI_file_shelf
        flag = gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT
        msgBox = gtk.MessageDialog(None, flag, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO, mess1)
        yes = msgBox.run()
        msgBox.destroy()
        if not yes == gtk.RESPONSE_YES:  return

        # write in Gimp_to-Inkscape data file
        GTI_f = open(GTI_file_shelf, 'w')
        GTI_f.write(repr(GTI_args))
        GTI_f.close()


### GIMP registering & UI ######################################################

register(
    "open-path-in-inkscape",
    _("This plug-in opens: active or all GIMP vectors in Inkscape, then can ")\
    +_("acquire and/or render them.\n")\
    +_("------------------------ Instructions: -------------------------\n")\
    +_("Before in GIMP, select the active path & its name (may use special characters).\n")\
    +_("After start, this name+'.svg' will be the save file for vector exchange.\n")\
    +_("In Inkscape: save changes and close. This return to GIMP and end the plug-in.\n")\
    +"------------------------------------------------------------------\n"\
    +_("Plug-in file-name:\n")+whoiam,
    "Open active or all vector layers to Inkscape and return to GIMP with a vectors\n"\
        +"re-acquisition and/or rendering possibility- V 0.4",
    "MareroQ and rob_brz1 (at gimpchat.com)",
    "GPLv3",
    "2016",
    _("Connection to Inkscape &..."),
    "RGB*,GRAY*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_VECTORS, "filename", "Start of vector exchange file", None),
        (PF_FILENAME, "ink", _("Inkscape executable file: "), GTI_args[0]),
        # change next for specifying exchange filename and create it
        #(PF_STRING, "xchange_utf", _("Exchange file-name:"), GTI_args[1]),
        (PF_DIRNAME, "outputFolder", _("Exchange directory:"), GTI_args[1]),
        (PF_OPTION, "sel", _("Vectors exported?"), GTI_args[2], (_("All paths"),_("Active path"))),
        (PF_OPTION, "schel", _("Return to GIMP with:"), GTI_args[3], (_("Nothing"),_("SVG vectors"),\
            _("SVG rendering"),_("Vectors + Render"))),
        (PF_BOOL, "rem_1", _("Delete SVG file? "), GTI_args[4])
    ],
    [],
    open_path_in_inkscape,
    menu = GTI_menu_path,
    domain = (whoiamName, locale_user_dir)
    )

main()
